#!/usr/bin/env python3

import re
from hashlib import md5

class QueryNotFound(Exception):
    def __init__(self, msg):
        self.message = msg

class Language(object):
    def __init__(self, code, name, about):
        if type(code) != str:
            raise TypeError('"code" argument is supposed to be str, not {0}.'.format(type(code)))
        if type(name) != str:
            raise TypeError('"name" argument is supposed to be str, not {0}.'.format(type(name)))
        if type(about) != dict:
            raise TypeError('"about" argument is supposed to be dict, not {0}.'.format(type(about)))
        
        self.__id__ = md5('{0}{1}{2}'.format(code, name, str(about)).encode('utf-8')).hexdigest().upper()
        self.__this__ = code
        self.__name__ = name
        self.__about__ = about
    
    def __call__(self):
        return {'type' : 'LANGUAGE', 'id' : self.__id__, 'this' : self.__this__, 'name' : self.__name__, 'about' : self.__about__}

class Text(object):
    def __init__(self, text, language, tags, about):
        if type(text) != str:
            raise TypeError('"text" argument is supposed to be str, not {0}.'.format(type(text)))
        if type(tags) != list:
            raise TypeError('"tags" argument is supposed to be list, not {0}.'.format(type(tags)))
        if type(language) != Language:
            raise TypeError('"language" argument is supposed to be corpse.obj.Language, not {0}.'.format(type(language)))
        for i in tags:
            if type(i) != Tag:
                raise TypeError('tags[{0}] element is supposed to be corpse.obj.Tag, not {1}.'.format(str(tags.index(i)), type(i)))
        if type(about) != dict:
            raise TypeError('"about" argument is supposed to be a dict, not {0}.'.format(type(about)))
        
        self.__id__ = md5('{0}{1}{2}'.format(text, str(tags), str(about)).encode('utf-8')).hexdigest().upper()
        self.__this__ = text
        self.__tags__ = tags
        self.__about__ = about
        
        self.__history__ = {}
    
    def __call__(self, excluding_this = True):
        if excluding_this == True:
            return {'type' : 'TEXT', 'id' : self.__id__, 'tags' : self.__tags__, 'about' : self.__about__}
        else:
            return {'type' : 'TEXT', 'id' : self.__id__, 'this' : self.__this__, 'tags' : self.__tags__, 'about' : self.__about__}
    
    def search(self, query):
        if type(query) != str:
            raise TypeError('"query" argument is supposed to be str, not {0}.'.format(type(query)))
        
        found = None
        samples = []
        
        if query in self.__history__: found = True
        else: found = False
        
        if found == True:
            samples = self.__history__[query]
        else:
            samples = re.findall(query, self.__this__)
            self.__history__[query] = samples
        
        del query, found
        return samples

class Tag(object):
    def __init__(self, tag):
        if type(tag) != str:
            raise TypeError('"tag" argument is supposed to be str, not {0}'.format(type(tag)))
        
        self.__id__ = md5('{0}'.format(tag).encode('utf-8')).hexdigest().upper()
        self.__this__ = tag